﻿console.log("Loading dialog module");

let electron = require("electron");
let dialog = electron.dialog;
let ipcMain = electron.ipcMain;
let BrowserWindow = electron.BrowserWindow;
let app = electron.app;

ipcMain.handle("prompt-open-file", async (event, browserWindowId, prompt) => {
    try {
        let window = BrowserWindow.fromId(browserWindowId);

        let filters = [];
        if (prompt.includeBlockpad) {
            if (prompt.includeExcel) {
                filters.push({
                    name: "Blockpad files and Excel spreadsheets",
                    extensions: ["bpad", "xls", "xlsx", "xlsm"]
                });
            }
            else {
                filters.push({
                    name: "All supported files",
                    extensions: ["bpad", "xmcd", "mcdx", "sm", "csv"]
                });
                filters.push({
                    name: "Blockpad files",
                    extensions: ["bpad"]
                });
                filters.push({
                    name: "Mathcad worksheets",
                    extensions: ["xmcd", "mcdx"]
                });
                filters.push({
                    name: "SMath files",
                    extensions: ["sm"]
                });
                filters.push({
                    name: "CSV (comma-delimited)",
                    extensions: ["csv"]
                });
            }

            if (prompt.includeAll) {
                filters.push({
                    name: "All files",
                    extensions: ["*"]
                })
            }
        }
        else if (prompt.includeImages) {
            if (prompt.includePdf) {
                filters.push({
                    name: "Images or PDFs",
                    extensions: ["jpg", "jpeg", "webp", "png", "gif", "svg", "pdf"]
                });
            }
            else {
                filters.push({
                    name: "Images",
                    extensions: ["jpg", "jpeg", "webp", "png", "gif", "svg"]
                });
            }

            if (prompt.includeAll) {
                filters.push({
                    name: "All files",
                    extensions: ["*"]
                })
            }
        }
        else if (prompt.includePdf) {
            filters.push({
                name: "PDFs",
                extensions: ["pdf"]
            });
        }
        else {
            filters.push({
                name: "All files",
                extensions: ["*"]
            })
        }

        let result = await dialog.showOpenDialog(window, {
            filters: filters,
            properties: ["openFile"],
            title: prompt.title
        });
        return result;
    }
    catch (ex) {
        console.log(ex);
        return ({ error: ex.toString() });
    }
});

ipcMain.handle("prompt-save-file", async (event, browserWindowId, prompt) => {
    try {
        let window = BrowserWindow.fromId(browserWindowId);

        let filters = [];
        if (prompt.includePdf) {
            filters.push({
                name: "PDF file",
                extensions: ["pdf"]
            });
        }
        else if (prompt.includeExcel) {
            filters.push({
                name: "Excel file",
                extensions: ["xlsx"]
            });
        }
        else {
            filters.push({
                name: "Blockpad file",
                extensions: ["bpad"]
            });
        }

        let result = await dialog.showSaveDialog(window, {
            filters: filters,
            properties: ["openFile"],
            title: prompt.title,
            defaultPath: prompt.fileName ?? undefined
        });
        return result;
    }
    catch (ex) {
        console.log(ex);
        return ({ error: ex.toString() });
    }
});

ipcMain.handle("handle-recent-file", (event, path) => {
    app.addRecentDocument(path);
});
