const { app, BrowserWindow, ipcMain, dialog, session } = require('electron');
const mainWindow = require('./main-window.js');
require("./dialogs.js");
const localServer = require("./local-server.js");

console.log(process.argv);

localServer.launch();

let isReady = false;

app.whenReady().then(async () => {
    isReady = true;
    mainWindow.createWindow();
});

let quitting = false;

app.on('window-all-closed', () => {
    if (process.platform !== 'darwin' || quitting) {
        app.quit()
    }
})

app.on('activate', () => {
    if (isReady && BrowserWindow.getAllWindows().length === 0) {
        mainWindow.createWindow()
    }
});

app.on("before-quit", () => {
    quitting = true;
});

ipcMain.handle("prevent-quit", event => {
    quitting = false;
});

let isLoaded = false;
let filesToLoad = [];

app.on("open-file", (event, path) => {
    event.preventDefault();
    if (isLoaded)
        doOpenFile(path);
    else
        filesToLoad.push(path);
});

app.on("will-finish-launching", () => {
    isLoaded = true;
    for (let path of filesToLoad)
        doOpenFile(path);
    filesToLoad = [];
});

function doOpenFile(path) {
    mainWindow.newTab(`/app/workspace?openFile=${path}`);
}