const fs = require('fs');
const path = require('path');
const os = require('os');

let confirmedPort = -1;

function getAppPort() {
    if (confirmedPort > 0)
        return confirmedPort;

    const fileContent = fs.readFileSync(getConfigFilePath(), 'utf-8');
    let lines = fileContent.split('\n').map(l => l.trim());

    for (let l of lines) {
        let parts = l.split("=");
        if (parts.length === 2 && parts[0] == "port") {
            let r = parseInt(parts[1]);
            return r;
        }
    }

    throw "No port found in config file";
}

function reset() {
    fs.unlink(getConfigFilePath(), err => { });
}

function getConfigFilePath() {
    return path.join(getAppDataDir(), "electron.config");
}

function isValid(port) {
    if (!isNaN(port) && Number.isInteger(port) && port >= 80) {
        return true;
    } else {
        return false;
    }
}

function getAppDataDir() {
    if (process.platform === 'win32') {
        return path.join(process.env.LOCALAPPDATA, "Blockpad");
    } else if (process.platform === 'darwin') {
        return path.join(os.homedir(), 'Library/Application Support/Blockpad/Blockpad Internal');
    } else {
        return path.join(os.homedir(), '.local/share/Blockpad');
    }
}

function confirmPort(port) {
    confirmedPort = port;
}

exports.getAppPort = getAppPort;
exports.reset = reset;
exports.isValid = isValid;
exports.confirmPort = confirmPort;