﻿import "./js/helpers.js";
import "./js/menus.js";
import "./js/popups.js";
import "./js/pointers.js";
import "./js/keyboard.js";
import "./js/textbox.js";
import "./js/inputAids.js";
import "./js/virtualDom.js";
import "./js/tableControls.js";
import "./js/fonts.js";
import "./js/drag-drop.js";
import "./js/loading-messages.js";
import "./js/requests.js";
import "./js/layout-window.js";
import { sendUpdatedToolbarMetrics } from "./js/toolbar.js";
import "./js/tooltips.js";
import "./js/dpi-handler.js";
import * as clipboard from "./js/clipboard.js";

window.blockpad = window.blockpad || {};
window.blockpad.clipboard = clipboard;

window.initWorkspace = workspace => {
    window.appWorkspace = workspace;
    sendUpdatedToolbarMetrics();
    // to do: send this again in callback if it changes
    return getInitWorkspaceInfo();
};

function getInitWorkspaceInfo() {
    return ({
        "isDarkMode": window.getComputedStyle(document.body).colorScheme === "dark"
    });
}

document.addEventListener("contextmenu", e => {
    if (!document.documentElement.classList.contains("app"))
        return;
    if (!(e.target?.nodeName === "INPUT" || e.target?.nodeName === "TEXTAREA")) {
        e.preventDefault();
    }
});

window.scrollToBottomOfElement = function (element) {
    setTimeout(() => element.scrollTop = element.scrollHeight, 0);
};

var okToRun = true;

window.run = handle => {
    if (okToRun) {
        okToRun = false;
        appWorkspace.invokeMethodAsync("Run", handle);
        setTimeout(() => okToRun = true, 10);
    }
};

window.logInClient = v => {
    console.log(v);
};

window.copyText = function (target) {
    let parentNode = target.parentNode;
    let input = firstOrDefault(parentNode.childNodes, n => n.nodeName === "INPUT");
    if (input) {
        input.select();
        document.execCommand("copy");
        setTimeout(() => target.focus(), 50);
    }
};

window.downloadFile = url => {
    const anchorElement = document.createElement('a');
    anchorElement.href = url;
    //anchorElement.download = url;
    anchorElement.click();
    anchorElement.remove();
};