﻿export async function getClipboardValue() {
    let clipboardItem = await navigator.clipboard.read();
    return await getClipboardData(clipboardItem);
};

async function getClipboardData(clipboardItems) {
    let result = {};
    for (let item of clipboardItems) {
        for (let type of item.types) {
            if (type === "text/plain" || type === "text/html") {
                let blob = await item.getType(type);
                result[type] = await blob.text();
                // to do: for long text, share as resource like for images
            }
            else if (type.startsWith('image/png')) {
                let blob = await item.getType(type);
                let id = crypto.randomUUID();
                let path = `/app/temp-resource?id=${id}`;
                // to do: for wasm version, use alternate way to send data
                let response = await fetch(path, {
                    method: "POST",
                    body: blob
                });
                if (response.ok) {
                    result[type] = {
                        type: "resource",
                        path: path
                    };
                }
            }
        }
    }
    return result;
}

export async function setClipboardValue(data) {
    let items = {};
    for (let key in data) {
        try {
            let value = data[key];
            if (typeof value === "string") {
                if (key === "text/html") {
                    let sanitizedHtml = getSanitizedHtml(value);
                    items["text/html"] = new Blob(
                        [sanitizedHtml],
                        {
                            type: "text/html; charset=UTF-8"
                        }
                    );
                }
                else if (key === "text/plain") {
                    items["text/plain"] = new Blob(
                        [value],
                        {
                            type: "text/plain; charset=UTF-8"
                        }
                    );
                }
            }
            else if (typeof value === "object") {
                let type = value["type"];
                if (type === "resource") {
                    let url = value["path"];
                    let response = await fetch(url);
                    let blob = await response.blob();
                    if (blob.type === "image/png") {
                        items["image/png"] = blob;
                    }
                }
            }
        }
        catch
        {
            console.log(`Error getting clipboard format ${key}`);
        }
    }
    let clipboardItem = new ClipboardItem(items);
    await navigator.clipboard.write([clipboardItem]);
};

function getSanitizedHtml(html) {
    const parser = new DOMParser();
    const doc = parser.parseFromString(html, 'text/html');
    return doc.documentElement.outerHTML;
}