﻿
window.doSafe = action => {
    try {
        if (action && typeof action === "function")
            action();
    }
    catch (ex) {

    }
};

window.openWindow = function (path) {
    window.open(withParams(path, {"lang": getLang()}), '_blank');
};

window.setTitle = function (title) {
    document.title = title;
};

window.getEventTarget = function (e) {
    if (e && e.srcElement)
        return e.srcElement;
    else if (e && e.currentTarget)
        return e.currentTarget;
    else
        return undefined;
};

window.isAnscestorOf = function (anscestor, node) {
    if (node === document)
        node = document.body;
    if (anscestor === document)
        anscestor = document.body;
    return getParentWithFilter(node, n => n === anscestor) !== null;
};

window.isSelfOrAnscestorOf = function (anscestor, node) {
    if (node === document)
        node = document.body;
    if (anscestor === document)
        anscestor = document.body;
    return getSelfOrParentWithFilter(node, n => n === anscestor) !== null;
};

window.getSelfOrParentWithFilter = function (element, filter) {
    if (element)
        for (var n = element; n; n = n.parentNode) {
            if (filter(n))
                return n;
            else if (n.nodeName === "HTML")
                break;
        }
    return null;
};

window.getParentWithFilter = function (element, filter) {
    if (element)
        return getSelfOrParentWithFilter(element.parentNode, filter);
    else
        return null;
};

window.getFirstChild = function (element, filter) {
    for (var child = element.firstChild; child; child = child.nextSibling)
        if (filter(child))
            return child;
    return null;
};

window.clearChildren = function (element) {
    element.innerHTML = "";
};

window.getDescendants = function* (root, enterTest, filter) {
    let stackItem = n => ({ node: n, children: getChildren(n) });

    var stack = [stackItem(root)];
    do {
        let item = stack[stack.length - 1];
        let enteringChild = false;
        while (true) {
            let r = item.children.next();
            if (!r.value)
                break;
            let n = r.value;
            if (filter(n))
                yield n;
            if (enterTest(n)) {
                stack.push(stackItem(n));
                enteringChild = true;
                break;
            }
            if (r.done)
                break;
        }
        if (!enteringChild)
            stack.pop();
    }
    while (stack.length > 0);
};

window.getChildren = function* (n) {
    if (n && n.childNodes)
        for (let child of n.childNodes)
            yield child;
    else if (n && n.children)
        for (let child of n.children)
            yield child;
};

window.firstOrDefault = function (list, condition) {
    for (var item of list) {
        try {
            if (condition === undefined || condition(item))
                return item;
        }
        catch {}
    }
};

window.any = function (list, condition) {
    if (list)
        for (var item of list)
            if (condition === undefined || condition(item))
                return true;
    return false;
};

window.getIndexOf = function (list, condition) {
    let i = 0;
    if (list) {
        for (let item of list) {
            if (condition(item))
                return i;

            i++;
        }
    }

    return -1;
};

window.lastOrDefault = function (list, condition) {
    var r;
    if (typeof condition !== "function")
        condition = n => true;
    if (list.values && typeof list.values[Symbol.iterator] === "function")
        list = list.values;
    for (var item of list)
        if (condition(item))
            r = item;
    return r;
};

window.removeAll = function (array, condition) {
    for (var i = array.length - 1; i >= 0; i--)
        if (condition(array[i]))
            array.splice(i, 1);
};

window.getSafe = function () {
    var r = arguments[0];
    for (var i = 1; i < arguments.length; i++) {
        if (!r)
            return undefined;

        try {
            var arg = arguments[i];
            if (typeof arg === "string")
                r = r[arg];
            else if (typeof arg === "function")
                r = arg(r);
            else
                return undefined;
        }
        catch (ex) {
            return undefined;
        }
    }
    return r;
};

window.cloneWithEvents = function (element) {
    let result = element.cloneNode(true);
    let attachEvent = eventName => result.addEventListener(eventName, e => {
        let target = getOriginalElement(e.target, result, element);
        if (target && isSelfOrAnscestorOf(element, target)) {
            if (eventName === "input")
                target.value = e.target.value;
            let event = document.createEvent("Event");
            event.initEvent(eventName, true, true);
            target.dispatchEvent(event);
        }
    });
    attachEvent("input");
    return result;
};

function getOriginalElement(cloneElement, cloneRoot, originalRoot) {
    if (cloneElement === cloneRoot)
        return originalRoot;

    if (cloneRoot && originalRoot) {
        let cloneParent = cloneElement.parentNode;
        let originalParent = cloneParent === cloneRoot ? originalRoot : getOriginalElement(cloneParent, cloneRoot, originalRoot);
        if (originalParent) {
            let cloneIndex = -1;
            let i = 0;
            cloneParent.childNodes.forEach(n => {
                if (n.tagName) {
                    if (n === cloneElement)
                        cloneIndex = i;
                    i++;
                }
            });
            if (cloneIndex >= 0) {
                i = 0;
                let result;
                originalParent.childNodes.forEach(n => {
                    if (n.tagName && !result) {
                        if (i === cloneIndex && nodesMatch(n, cloneElement))
                            result = n;
                        i++;
                    }
                });
                return result;
            }
        }
    }
}

function nodesMatch(a, b) {
    return a && b && a.tagName === b.tagName;
}

// returns point transform function (given {x, y}, return {x, y})
window.getTransform = function (fromElement, toElement) {
    let fromPath = getOffsetPath(fromElement);
    let toPath = getOffsetPath(toElement);
    let commonAnscestor = firstOrDefault(fromPath, n => any(toPath, n2 => n2 === n));
    if (commonAnscestor) {
        return pt => {
            let r = pt;
            for (let i = 0; i < fromPath.length; i++) {
                let n = fromPath[i];
                if (n === commonAnscestor)
                    break;
                r.x += n.offsetLeft;
                r.y += n.offsetTop;
                r.x -= n.scrollLeft;
                r.y -= n.scrollTop;
            }
            for (let i = 0; i < toPath.length; i++) {
                let n = toPath[i];
                if (n === commonAnscestor)
                    break;
                r.x -= n.offsetLeft;
                r.y -= n.offsetTop;
                r.x += n.scrollLeft;
                r.y += n.scrollTop;
            }
            return r;
        };
    }
};

function getOffsetPath(element) {
    let r = [element];
    for (let n = element; n; n = n.offsetParent) {
        if (n.offsetParent)
            r.push(n.offsetParent);
    }
    return r;
}

window.getPointerPath = function (element) {
    let r = [];
    let n = element;
    while (n && n.nodeName && n.nodeName !== "HTML") {
        r.push({ className: n.className ? n.className : "" });
        n = n.parentNode;
    }
    return r;
};

window.resetSelectValue = function (element) {

};

window.updateCursor = function (element, value) {
    element.style.cursor = value;
};

window.setContent = function (id, value) {
    let valueString = BINDING.conv_string(value);
    let element = document.getElementById("svg-" + id);
    element.innerHTML = valueString;
};

window.whenElementWithIdReady = async id => {
    for (var i = 0; i < 30; i++) {
        let element = document.getElementById(id);
        if (element)
            return true;
        await sleep(100);
    }
    return false;
};

const sleep = (milliseconds) => {
    return new Promise(resolve => setTimeout(resolve, milliseconds))
}

window.withParams = function (url, paramValues) {
    var urlObject = new URL(url, location.origin);
    for (let key in paramValues) {
        let value = paramValues[key];
        if (value)
            urlObject.searchParams.set(key, value)
    }
    if (urlObject.origin === location.origin) {
        return urlObject.pathname + urlObject.search;
    }
    else {
        return urlObject.href;
    }
}

window.getLang = function () {
    const urlParams = new URLSearchParams(window.location.search);
    return urlParams.get("lang");
}