﻿
window.keepMenusOpenOnBlur = false;

window.addEventListener("blur", e => {
    window.isShiftDown = false;
    window.isCtrlDown = false;

    if (!window.keepMenusOpenOnBlur)
        window.hideMenusOffTargetPath(document.body);
});

window.addEventListener("focus", e => {
    window.isShiftDown = false;
    window.isCtrlDown = false;
});

let lastActiveElement;

if (document.documentElement.classList.contains("app")) {

    window.addEventListener("focus", e => {
        console.log("focused " + document.activeElement);

        if (!applyingFocus && !mayFocusElement(document.activeElement) && mayFocusElement(lastActiveElement)) {
            if (lastActiveElement.hasPointerCapture(document.activeElement)) {
                lastActiveElement.focus();
            }
        }
        else {
            lastActiveElement = document.activeElement;
            // to do: notify application
        }
    }, true);

    window.addEventListener("pointerup", e => {
        if (!mayFocusElement(document.activeElement) && mayFocusElement(lastActiveElement)) {
            lastActiveElement.focus();
        }
    });

    document.body.addEventListener("focusout", e => {
        console.log("focus out " + e.target + ", now " + document.activeElement);

        //if (!applyingFocus && document.activeElement.tagName === "BODY") {
        //    let _lastActiveElement = lastActiveElement;
        //    setTimeout(() => applyFocus(_lastActiveElement));
        //}
    });

    function mayFocusElement(element) {
        if (!element || element.tagName === "BODY")
            return false;
        else if (element.tagName === "BUTTON")
            return document.activeElement.tabIndex !== -1;
        else
            return true;
    }
}

let applyingFocus = false;

window.applyFocus = element => {
    if (applyingFocus)
        return;

    try {
        applyingFocus = true;
        let r = element.focus();
    }
    finally {
        applyingFocus = false;
    }
};

window.sendTextInput = text => {
    if (lastActiveElement && lastActiveElement.focus && isAnscestorOf(document.body, lastActiveElement)) {
        lastActiveElement.focus();
    }

    setTimeout(() => {
        keyDownHandlerCount++;
        setTimeout(() => keyDownHandlerCount--);

        if (text && text.length) {
            for (let i = 0; i < text.length; i++) {
                let data = { key: text.substring(i, 1), bubbles: true, cancelable: true, view: window };
                if (document.activeElement && (document.activeElement.nodeName === "INPUT" || document.activeElement.nodeName === "TEXTAREA")) {
                    doHandleInputEventNative(data);
                }
                else {
                    document.activeElement.dispatchEvent(new KeyboardEvent("keydown", data));
                    document.activeElement.dispatchEvent(new KeyboardEvent("keypress", data));
                    document.activeElement.dispatchEvent(new KeyboardEvent("keyup", data));
                }
            }
        }
    });
};

window.isCtrlDown = false;
window.isShiftDown = false;
window.isAltDown = false;

let keyDownHandlerCount = 0;
let keyDownHandled = false;

window.isHandlingKeyDown = () => {
    let result = keyDownHandlerCount > 0 || keyDownHandled;
    keyDownHandled = false;
    return result;
};

window.addEventListener("keydown", e => {
    console.log("Keydown Value: " + e.target.value);
    keyDownHandled = false;

    if (e.key === "Control")
        window.isCtrlDown = true;
    else if (e.key === "Shift")
        window.isShiftDown = true;
    else if (e.key === "Alt")
        window.isAltDown = true;

    //if (!window.appWorkspace)
    //    return;

    if (ignoreKeyEvent(e))
        return;

    let _e = getInputEvent(e);
    if (!pendingKeyEventGroups && tryHandleInputEventNative(_e)) {
        //console.log(`KeyPress ${_e.key} - native`); // temp
        if (trySpecialNativeEvent(_e)) {
            
            e.preventDefault();
        }
    }
    else {
        keyDownHandled = true;
        keyDownHandlerCount++;
        setTimeout(() => keyDownHandlerCount--, 0);

        //console.log(`KeyPress ${_e.key} - back-end`); // temp
        let handleNativeToo = !window.isCtrlDown && !window.isShiftDown && document.activeElement.classList.contains("layout-window-input");
        if (!handleNativeToo)
            e.preventDefault();

        if (e.key === "Enter" && (document.activeElement.nodeName === "INPUT" || document.activeElement.nodeName === "TEXTAREA"))
            document.activeElement.dispatchEvent(new Event("change"));
        processInputEvents([_e]);
    }

    if (e.key === "Backspace" && !e.defaultPrevented && (!document.activeElement || !(document.activeElement.nodeName === "INPUT" || document.activeElement.nodeName === "TEXTAREA")))
        e.preventDefault();
}, true);

window.addEventListener("keyup", e => {
    console.log("Keyup Value: " + e.target.value);
    if (e.key === "Control")
        window.isCtrlDown = false;
    else if (e.key === "Shift")
        window.isShiftDown = false;
    else if (e.key === "Alt")
        window.isAltDown = false;
});

function ignoreKeyEvent(e) {
    switch (e.key) {
        case "F12":
        case "Shift":
        case "Ctrl":
        case "Alt":
        case "OS":
            return true;
        default:
            return false;
    }
}

let pendingKeyEventGroups = 0;

function processInputEvents(events) {
    var firstEvent = firstOrDefault(events);
    if (firstEvent) {
        let focusElement = getElementCode(document.activeElement);
        events.forEach(e => e.focusElement = focusElement);
        pendingKeyEventGroups++;
        let done = false;
        function markDone() {
            if (!done) {
                done = true;
                console.log("done handling " + firstEvent.key); // temp
                pendingKeyEventGroups--;
            }
        }
        setTimeout(markDone, 2000);
        appWorkspace.invokeMethodAsync("HandleKeyEvents", events)
            .then(() => setTimeout(markDone))
            .catch(() => setTimeout(markDone));
    }
}

window.processInputEvents = processInputEvents;

function getInputEvent(e) {
    return ({
        focusElement: getElementCode(document.activeElement),
        type: e.type,
        key: e.key,
        ctrlKey: e.ctrlKey,
        altKey: false, // to do: for web app on Windows, try to tell the difference between Alt and AltGr
        shiftKey: e.shiftKey,
        metaKey: e.metaKey,
        charCode: e.charCode,
        which: e.which || e.keyCode
    });
}

function inputEventsEqual(a, b) {
    return a.type === b.type &&
        a.key === b.key &&
        a.keyCode === b.keyCode &&
        a.ctrlKey === b.ctrlKey &&
        a.altKey === b.altKey &&
        a.shiftKey === b.shiftKey &&
        a.metaKey === b.metaKey &&
        a.which === b.which;
}

function getElementCode(element) {
    return `${element.nodeName}-${element.id}`;
}

window.setKeyOverrides = (element, keysToOverride) => {
    if (typeof element === "object" && element)
        element.keysToOverride = keysToOverride;
};

function tryHandleInputEventNative(e) {
    if (!window.appWorkspace)
        return true;

    if (window.isAltDown) {
        if (e.key === "0" || e.key === "1" || e.key === "2" || e.key === "3" || e.key === "4" || e.key === "5" || e.key === "6" || e.key === "7" || e.key === "8" || e.key === "9") {
            return true;
        }
    }

    switch (e.key) {
        case "l":
            if (e.ctrlKey || e.metaKey)
                return true;

            break;
        case "F5":
            if (e.ctrlKey || e.metaKey)
                return true;

            let reconnectModal = document.getElementById("components-reconnect-modal");
            if (reconnectModal && getComputedStyle(reconnectModal).display !== "none")
                return true;

            break;
        case "F12":
            return true;
        case "h":
        case "q":
            if (isMac() && e.metaKey) {
                return true;
            }
            break;
    }

    switch (getSafe(document.activeElement, "nodeName")) {
        case "BODY":
        case "DIV":
            return false;
        default:
            if (document.activeElement.classList && document.activeElement.classList.contains("layout-window-input")) {
                return false;
            }

            switch (e.key) {
                case "F1":
                case "F2":
                case "F3":
                case "F4":
                case "F5":
                case "F6":
                case "F7":
                case "F8":
                case "F9":
                case "F10":
                case "F11":
                case "Escape":
                    return false;
                case "c":
                case "x":
                case "v":
                case "a":
                case "z":
                case "y":
                    return true;
                case "Enter":
                    return !overridesKey(document.activeElement, e);
                case "ArrowLeft":
                case "ArrowRight":
                case "ArrowUp":
                case "ArrowDown":
                case "Tab":
                    return !overridesKey(document.activeElement, e);
                case ".":
                    return !e.ctrlKey && !suggestionsListOpen(document.activeElement);
                case "(":
                    return !e.ctrlKey && !suggestionsListOpen(document.activeElement);
                default:
                    return e.key === "Backspace" || e.key === "Delete" || !e.ctrlKey || e.key === "l";
            }
    }
}

function isMac() {
    let params = new URLSearchParams(document.location.search);
    let name = params.get("mac");
    return !!name;
}

function overridesKey(element, event) {
    return element && element.keysToOverride && any(element.keysToOverride, k => k.key === event.key && k.ctrl === event.ctrlKey && k.shift == event.shiftKey && k.alt === event.altKey);
}

// only call this when an input event is unfinished returning from back-end
function doHandleInputEventNative(e) {
    if (trySpecialNativeEvent(e))
        return;

    switch (getSafe(document.activeElement, "nodeName")) {
        case "INPUT":
        case "TEXTAREA":
            if (e.key === "Backspace") {

            }
            else if (e.key.length === 1) {
                let a = document.activeElement.selectionStart;
                let b = document.activeElement.selectionEnd;
                let oldVal = document.activeElement.value;
                document.activeElement.value = oldVal.substring(0, a) + e.key +
                    oldVal.substring(b);
                document.activeElement.selectionStart = a + 1;
                document.activeElement.selectionEnd = a + 1;
                let event = new Event('input', {
                    'bubbles': true,
                    'cancelable': true
                });

                document.activeElement.dispatchEvent(event);
            }
            break;
    }
}

function trySpecialNativeEvent(e) {
    let hasEqualsToggle = getSafe(document.activeElement, "previousElementSibling", "classList", classList => classList.contains("formulaIcon") && classList.contains("toggle"));
    if (hasEqualsToggle) {
        let equalsToggle = document.activeElement.previousElementSibling;
        if (e.key === "=") {
            if (document.activeElement.selectionStart == 0 && equalsToggle.style.display === "none") {
                equalsToggle.style.display = "block";

                let event = new Event('input', {
                    'bubbles': true,
                    'cancelable': true
                });

                document.activeElement.dispatchEvent(event);
                return true;
            }
        }
        else if (e.key === "Backspace") {
            if (document.activeElement.selectionStart === 0 && document.activeElement.selectionEnd === document.activeElement.selectionStart && equalsToggle.style.display !== "none") {
                equalsToggle.style.display = "none";

                let event = new Event('input', {
                    'bubbles': true,
                    'cancelable': true
                });

                document.activeElement.dispatchEvent(event);
                return true;
            }
        }
    }

    if (e.key === "Enter") {
        let element = document.activeElement;
        if (getSelfOrParentWithFilter(element, n => n.classList?.contains("textBoxWrapper"))) {
            if (e.altKey) {
                if (element.nodeName === "TEXTAREA") {
                    let selStart = element.selectionStart;
                    let selEnd = element.selectionEnd;
                    let insertValue = "\n";
                    let newValue = element.value.substring(0, selStart) + insertValue + element.value.substring(selEnd);
                    element.value = newValue;
                    element.selectionStart = selStart + insertValue.length;
                    element.selectionEnd = element.selectionStart;
                    updateTextboxSize(element);
                }
                return true;
            }
            else {
                let multiline = element.parentNode?.classList?.contains("multiline");
                if (!multiline) {
                    document.activeElement.dispatchEvent(new Event("change"));
                    return true;
                }
            }
        }
    }

    return false;
}