﻿
window.registerLayoutWindow = function (elements, target) {
    let handleNative = false;

    let layoutWindow = getSelfOrParentWithFilter(firstOrDefault(Object.values(elements)), n => n.classList && n.classList.contains("layoutWindow"));

    for (let key in elements) {
        let element = elements[key];
        if (element) {
            element.oncontextmenu = e => {
                setTimeout(() => target.invokeMethodAsync("ContextMenu", key, serializeEvent(e, element)));
            };
            element.onpointerdown = e => {
                handleNative = false;
                if (!handleNative) {
                    element.setPointerCapture(e.pointerId);
                    doSafe(updatePagesLayouts);
                    target.invokeMethodAsync("MouseDown", key, serializeEvent(e, element));
                }
            };
            element.onpointermove = e => {
                if (!e.buttons) {
                    handleNative = false;
                }
                if (!handleNative) {
                    doSafe(updatePagesLayouts);
                    sendMouseMove(key, element, e);
                }
            };
            element.onpointerup = e => {
                if (!handleNative) {
                    element.releasePointerCapture(e.pointerId);
                    doSafe(updatePagesLayouts);
                    target.invokeMethodAsync("MouseUp", key, serializeEvent(e, element));
                }
            };
            element.onpointerleave = e => {
                if (!handleNative) {
                    target.invokeMethodAsync("MouseLeave");
                }
            };
            element.onwheel = e => {
                if (e.ctrlKey) {
                    target.invokeMethodAsync("MouseWheel", key, serializeEvent(e, element));
                    e.preventDefault();
                    return false;
                }
            };
        }
    }

    function sendMouseMove(key, element, e) {
        target.invokeMethodAsync("MouseMove", key, serializeEvent(e, element));
    }

    let layoutWindowInput = layoutWindow.getElementsByClassName("layout-window-input")[0];
    if (layoutWindowInput) {
        layoutWindowInput.oninput = e => {
            let data = e.data;
            let handling = window.isHandlingKeyDown();
            if (!handling && data) {
                processInputEvents([{
                    focusElement: `${layoutWindowInput.nodeName}-${layoutWindowInput.id}`,
                    //type: e.type,
                    key: data,
                    ctrlKey: window.isCtrlDown,
                    altKey: false, // to do: for web app on Windows, try to tell the difference between Alt and AltGr
                    shiftKey: window.isShiftDown,
                    metaKey: false,
                    //charCode: e.charCode,
                    //which: e.which || e.keyCode
                }])
            }

            // to do: also handle if special mobile keyboard inputs, etc.
        };
    }
};

function updatePagesLayouts() {

}

window.registerScrollLayout = function (element, target) {
    if (!element)
        return;

    element.sendScrollInfo = () => sendScrollInfo(element, target);
    element.onscroll = e => sendScrollInfo(element, target);
    setTimeout(() => element.sendScrollInfo(), 10);
};

window.addEventListener("resize", () => {
    var scrollLayouts = document.getElementsByClassName("scrollElementLayout");
    for (let element of scrollLayouts) {
        var sendUpdate = element.sendScrollInfo;
        if (sendUpdate)
            sendUpdate();
    }
});

window.requestScrollInfo = function (element, target) {
    setTimeout(() => {
        sendScrollInfo(element, target);
    }, 10);
};

function sendScrollInfo(element, target) {
    updateRulerPlacement(element);

    let info = getScrollInfo(element);
    if (!scrollInfosEqual(info, element.lastScrollInfo)) {
        element.lastScrollInfo = info;
        target.invokeMethodAsync("SetScrollInfo", info);
    }
}

function updateRulerPlacement(element) {

}

function getScrollInfo(element) {
    var r = {
        viewportWidth: element.clientWidth,
        viewportHeight: element.clientHeight,
        xOffset: element.scrollLeft,
        yOffset: element.scrollTop,
        scrollAreaXOffset: 0,
        scrollAreaYOffset: 0,
        scrollAreaWidth: 0,
        scrollAreaHeight: 0,
        layoutWidth: element.clientWidth,
        layoutHeight: element.clientHeight
    };
    var scrollArea = firstOrDefault(getDescendants(element, n => true, n => n.className === "scrollArea"));
    if (scrollArea) {
        if (element.classList.contains("pagesLayoutContent")) {
            element.style.overflowX = getOverflow(element.clientWidth, scrollArea.offsetWidth);
            element.style.overflowY = getOverflow(element.clientHeight, scrollArea.offsetHeight);
        }
        r.xOffset -= scrollArea.offsetLeft;
        r.yOffset -= scrollArea.offsetTop;
        r.scrollAreaXOffset = scrollArea.offsetLeft;
        r.scrollAreaYOffset = scrollArea.offsetTop;
        r.scrollAreaWidth = scrollArea.offsetWidth;
        r.scrollAreaHeight = scrollArea.offsetHeight;
    }
    return r;
}

function getOverflow(layoutLength, contentLength) {
    if (layoutLength >= contentLength)
        return "hidden";
    else
        return "scroll";
}

function scrollInfosEqual(a, b) {
    if (!a || !b)
        return false;

    return a.viewportWidth === b.viewportWidth && a.viewportHeight === b.viewportHeight &&
        a.xOffset === b.xOffset && a.yOffset === b.yOffset;
}

window.resetCaretBlink = container => {
    setTimeout(() => {
        if (container) {
            let caret = firstOrDefault(container.getElementsByClassName("vertCaret"));
            if (!caret)
                caret = firstOrDefault(container.getElementsByClassName("horzCaret"));

            if (caret) {
                caret.classList.remove("blink");
                caret.classList.add("blink");
            }
        }
    }, 600);
};

window.setScrollOffset = (pagesLayout, xOffset, yOffset) => {
    if (!pagesLayout)
        return;

    pagesLayout.scrollTo(xOffset, yOffset);
};