﻿window.showContainedMenu = (element, onlyIfSiblingOpen, owner, keepOpenOnClick) => {
    if (!element || !element.classList)
        return;

    if (onlyIfSiblingOpen) {
        let container = element.parentNode.parentNode;
        let anySiblingsOpen = any(openContainedMenus, m => m.parentNode.parentNode === container);
        if (!anySiblingsOpen)
            return;
    }

    element.popupOwner = owner;
    element.keepOpenOnClick = keepOpenOnClick;
    element.classList.remove("hidden");
    element.classList.add("shown");
    openContainedMenus.push(element);
    notifyParentMenuVisible(element.parentNode);
    keepMenuInView(element);

    hideMenusOffTargetPath(element);
    element.parentNode.parentNode.childNodes.forEach(otherContainer => {
        if (otherContainer !== element.parentNode) {
            let otherMenu = firstOrDefault(otherContainer.childNodes, e => e.classList && e.classList.contains("menu"));
            if (otherMenu && otherMenu !== element && any(openContainedMenus, m => m === otherMenu))
                hideContainedMenu(otherMenu);
        }
    });

};

window.keepMenusInView = () => {
    openContainedMenus.forEach(keepMenuInView);
};

function notifyParentMenuVisible(parent) {
    parent.classList.add("openMenuHeader");
    if (parent.referenceElement) {
        let n = parent.referenceElement;
        n.classList.add("openMenuHeader");
        for (n = n.parentNode; n; n = n.parentNode) {
            if (n.referenceElement) {
                n = n.referenceElement;
                n.childNodes.forEach(n => {
                    if (n.classList && n.classList.contains("menuHeader"))
                        n.classList.add("openMenuHeader");
                });
            }
            if (n.classList && n.classList.contains("menuHeader"))
                n.classList.add("openMenuHeader");
        }
    }
}

window.hideMenusOffTargetPath = target => {
    let menusToHide = [];
    let button = getSelfOrParentWithFilter(target, n => n.tagName === "BUTTON");
    openContainedMenus.forEach(menu => {
        if ((button && button.parentNode === menu.parentNode) || (!menuOnPath(menu, target) /*&& !menu.keepOpenOnClick*/))
            menusToHide.push(menu);
    });
    menusToHide.forEach(hideContainedMenu);
    if (menusToHide.length > 0) {
        setTimeout(window.updatePopups);
    }
    return menusToHide.length > 0;
};

function menuOnPath(menu, target) {
    if (isAnscestorOf(menu.parentNode, target))
        return true;

    let altTarget = target?.parentNode?.referenceElement;
    if (altTarget) {
        for (let n of getMenuAnscestors(altTarget)) {
            if (n === menu)
                return true;
        }
    }

    return false;
}

window.hideContainedMenuWithOwner = owner => {
    let element = firstOrDefault(openContainedMenus, m => m.popupOwner === owner);
    if (element) {
        element.popupOwner = undefined;
        hideContainedMenu(element);
    }
}

function hideContainedMenu(element) {
    element.classList.add("hidden");
    element.classList.remove("shown");
    let index = openContainedMenus.indexOf(element);
    if (index >= 0)
        openContainedMenus.splice(index, 1);
    notifyParentMenuHidden(element.parentNode);
    if (window.appWorkspace && typeof element.popupOwner === "number" && element.popupOwner > 0)
        appWorkspace.invokeMethodAsync("HidingMenu", element.popupOwner);
}

function notifyParentMenuHidden(parent) {
    for (let n of getMenuAnscestors(parent)) {
        if (n.classList && n.classList.contains("openMenuHeader")) {
            if (!stillAnscestorOfOtherMenu(n))
                n.classList.remove("openMenuHeader");
        }
    }
}

function stillAnscestorOfOtherMenu(n) {
    for (let menu of openContainedMenus) {
        for (let menuAnscestor of getMenuAnscestors(menu.parentNode)) {
            if (n === menuAnscestor)
                return true;
        }
    }

    return false;
}

window.getMenuAnscestors = function* (firstParent) {
    for (let n = firstParent; n; n = n.parentNode) {
        yield n;
        if (n.referenceElement) {
            n = n.referenceElement;
            yield n;
        }
    }
};

window.hideContainedMenus = () => {
    for (let i = openContainedMenus.length - 1; i >= 0; i--) {
        let menu = openContainedMenus[i];
        hideContainedMenu(menu);
        openContainedMenus.splice(i, 1);
    }
};

var openContainedMenus = [];

window.showButtonMenu = button => {
    let parent = button.parentNode;
    let menu = parent.getElementsByClassName("menu")[0];
    if (menu) {
        menu.classList.remove("hidden");
        menu.classList.add("shown");
        parent.classList.add("openMenuHeader");
        openContainedMenus.push(menu);
    }
};

function keepMenuInView(menu) {
    if (!menu.parentNode.referenceElement)
        return;

    let rect = menu.parentNode.referenceElement.getBoundingClientRect();
    let menuRect = menu.getBoundingClientRect();
    let parentRect = menu.parentNode.getBoundingClientRect();
    let isVert = menu.style.top === "100%" || menu.style.bottom === "100%";
    let isHorz = menu.style.left === "100%" || menu.style.right === "100%";
    let offset = "100%";
    let offsetPlus2 = "calc(100% + 2px)"

    if (!isVert && !isHorz) {
        if (menu.style.top === offsetPlus2) {
            isVert = true;
            offset = offsetPlus2;
        }
        else if (menu.style.bottom === offsetPlus2) {
            isVert = true;
            offset = offsetPlus2;
        }
    }

    if (isVert) {
        var availHeightAbove = rect.top;
        var availHeightBelow = document.body.clientHeight - rect.height - rect.top;
        if (menuRect.height > availHeightBelow) {
            if (availHeightAbove > availHeightBelow) {
                menu.style.bottom = offset;
                menu.style.top = "unset";
                menu.style.maxHeight = `${availHeightAbove - 5}px`;
            }
            else {
                menu.style.bottom = "unset";
                menu.style.top = offset;
                menu.style.maxHeight = `${availHeightBelow - 5}px`;
            }
        }
        else {
            menu.style.bottom = "unset";
            menu.style.top = offset;
            menu.style.maxHeight = `${availHeightBelow - 5}px`;
        }
    }
    else if (isHorz) {

    }
    else {
        if (menuRect.bottom > document.body.clientHeight) {
            menu.style.bottom = `${-(document.body.clientHeight - parentRect.bottom)}px`;
            menu.style.top = "unset";
            menu.style.maxHeight = `${document.body.clientHeight}`;
        }
    }
}