﻿
window.addEventListener("click", e => {
    if (blockClicks) {
        blockClicks = false;
        e.preventDefault();
        e.stopPropagation();
        return;
    }

    var target = getEventTarget(e);
    var menuItemDef = getSafe(target, "itemDef");
    if (menuItemDef) {
        run(menuItemDef.handle);
        hideMenus();
        return;
    }

    let subItems = getSafe(target, "subItems");
    if (subItems) {
        showMenu();
        return;
    }
}, true);

var blockClicks = false;

var movingElement = null;
var movingRefX, movingRefY;

window.addEventListener("pointerdown", e => {
    let target = getEventTarget(e);
    let anyMenusHidden = hideMenusOffTargetPath(target);
    if (anyMenusHidden) {
        let button = getSelfOrParentWithFilter(target, n => n.tagName === "BUTTON");
        if (button) {
            e.preventDefault();
            e.stopPropagation();
            return false;
        }
    }

    let menuElement = getSelfOrParentWithFilter(target, n => getSafe(n, "className") === "menu" || getSafe(n, "className") === "dropDown");
    if (menuElement) {
        return;
    }

    if (window.anyMenusOpen && anyMenusOpen() && !window.preventHideMenusOnClick) {
        hideMenus();
        blockClicks = true;
        e.preventDefault();
        e.stopPropagation();
        return false;
    }

    let button = getSelfOrParentWithFilter(target, n => n.tagName === "BUTTON");
    let movableElement = getSelfOrParentWithFilter(target, n => n.classList && n.classList.contains("movable") && !n.classList.contains("inputRow"));
    if (movableElement && !button) {
        let a = movableElement.getBoundingClientRect();
        let b = movableElement.offsetParent.getBoundingClientRect();
        let titleHeight = 20;

        let h5 = movableElement.getElementsByTagName("h5")[0];
        if (h5) {
            titleHeight = h5.getBoundingClientRect().bottom - a.top;
        }
        else {
            let label = movableElement.getElementsByTagName("label")[0];
            if (label) {
                let labelSpan = label.firstElementChild;
                if (labelSpan && labelSpan.tagName === "SPAN") {
                    titleHeight = labelSpan.getBoundingClientRect().bottom - a.top;
                }
            }
        }

        if (e.clientY - a.top < titleHeight) {
            movingElement = movableElement;
            movingRefX = e.clientX - (a.left - b.left);
            movingRefY = e.clientY - (a.top - b.top);
            movingElement.setPointerCapture(e.pointerId);
            movingElement.classList.add("moving");
            return;
        }
    }

    if (button?.parentNode?.classList?.contains("dropDownWrapper")) {
        let dropDownWrapper = button.parentNode;
        let input = dropDownWrapper.getElementsByTagName("input")[0];
        if (input) {
            input.focus();
        }
    }
}, true);

window.addEventListener("pointerdown", e => {
    let target = getEventTarget(e);

    let elementToFocus = getSelfOrParentWithFilter(target, n => n.firstElementChild?.classList?.contains("layout-window-input"));
    if (elementToFocus) {
        elementToFocus.firstElementChild.focus();
        e.preventDefault();
        return false;
    }

    if (target?.tagName === "INPUT" || target?.tagName === "TEXTAREA" || target?.tagName === "SELECT" || target?.tagName === "OPTION" || target?.tagName === "BUTTON") {
        return true;
    }

    if (getSelfOrParentWithFilter(target, n => (n.tagName === "APP" || n.tagName === "app") && !document.body.classList?.contains("full-content"))) {
        if (hasPendingInput && (document.activeElement?.nodeName === "INPUT" || document.activeElement?.nodeName === "TEXTAREA")) {
            document.activeElement.dispatchEvent(new Event("change"));
        }

        //let isDraggable = !!getSelfOrParentWithFilter(target, n => n.draggable);
        //if (!isDraggable) {
        //    e.preventDefault();
        //    return false;
        //}
    }
});

let hasPendingInput = false;

window.addEventListener("input", e => {
    hasPendingInput = true;
}, true);

window.addEventListener("change", e => {
    hasPendingInput = false;
}, true);

window.addEventListener("pointermove", e => {
    if (movingElement) {
        movingElement.style.left = `${e.clientX - movingRefX}px`;
        movingElement.style.top = `${Math.max(0, e.clientY - movingRefY)}px`;
    }
});

window.addEventListener("pointerup", e => {
    if (blockClicks) {
        e.preventDefault();
        e.stopPropagation();
        setTimeout(() => blockClicks = false, 10);
    }

    if (movingElement) {
        movingElement.releasePointerCapture(e.pointerId);
        movingElement.classList.remove("moving");
        movingElement = null;
    }
});

var clickCountTimer = null;
window.clickCount = 0;
var clickX, clickY;

window.addEventListener("pointerdown", e => {
    if (clickCountTimer)
        clearTimeout(clickCountTimer);

    if (clickX === e.clientX && clickY === e.clientY)
        window.clickCount++;
    else
        window.clickCount = 1;

    clickX = e.clientX;
    clickY = e.clientY;

    clickCountTimer = setTimeout(() => {
        window.clickCount = 0;
        clickCountTimer = null;
    }, 500);
}, true);

let appElement = document.getElementsByTagName("app")[0];
if (appElement) {
    appElement.addEventListener("wheel", e => {
        let layoutWindow = getParentWithFilter(e.target, n => n.classList?.contains("layoutWindow"));
        if (!layoutWindow)
            return;

        if (e.ctrlKey) {
            e.preventDefault();
            e.stopPropagation();
        }
    }, false);
}

window.serializeEvent = function (e, element) {
    if (e) {
        var elementOffsetLeft = 0, elementOffsetTop = 0;
        let box = element.getBoundingClientRect();
        if (element) {
            elementOffsetLeft = element.offsetLeft;
            elementOffsetTop = element.offsetTop;
        }
        var o = {
            altKey: e.altKey,
            button: e.button,
            buttons: e.buttons,
            clientX: e.clientX - elementOffsetLeft,
            clientY: e.clientY - elementOffsetTop,
            ctrlKey: e.ctrlKey,
            deltaY: e.deltaY,
            detail: clickCount,
            metaKey: e.metaKey,
            movementX: e.movementX,
            movementY: e.movementY,
            offsetX: e.clientX - box.x,
            offsetY: e.clientY - box.y,
            pageX: e.pageX,
            pageY: e.pageY,
            screenX: e.screenX,
            screenY: e.screenY,
            shiftKey: e.shiftKey
        };
        return o;
    }
};
