﻿window.setPopupPlacement = (popupElement, referenceElement, textIndex) => {
    popupElement.referenceElement = referenceElement;
    popupElement.popupTextIndex = textIndex;
    popupElements.push(popupElement);

    popupElements.sort((a, b) => {
        if (any(getMenuAnscestors(a), n => n === b))
            return 1;
        else if (any(getMenuAnscestors(b), n => n === a))
            return -1;
        else
            return 0;
    });
    updatePopups();
};

function updatePopups() {
    let popupsToRemove = [];
    for (let i = 0; i < popupElements.length; i++) {
        let popupElement = popupElements[i];
        let referenceElement = popupElement.referenceElement;
        if (!isAnscestorOf(document.body, popupElement) || !isAnscestorOf(document.body, referenceElement)) {
            popupsToRemove.push(popupElement);
        } else {
            updatePopup(popupElement);
        }
    }
    popupsToRemove.forEach(popupElement => {
        let i = popupElements.indexOf(popupElement);
        if (i >= 0)
            popupElements.splice(i, 1);
        popupElement.classList.add("hidden");
        popupElement.classList.remove("shown");
    });
    keepMenusInView();
}

window.updatePopups = updatePopups;

window.updatePopup = (popupElement) => {
    let referenceElement = popupElement.referenceElement;
    let textIndex = popupElement.popupTextIndex;
    let style = window.getComputedStyle(referenceElement);

    if (!(style?.visibility === "hidden")) {
        //debugger;
    }

    let rect = getVisibleBoundingRect(referenceElement);

    if (rect.width <= 0 || rect.height <= 0 || style?.visibility === "hidden") {
        popupElement.classList.add("hidden");
        popupElement.classList.remove("shown");
    }
    else {
        popupElement.classList.remove("hidden");
        popupElement.classList.add("shown");

        if (referenceElement && typeof textIndex === "number" && textIndex >= 0) {
            let caretPosition = getCaretPosition(referenceElement, textIndex);
            rect.x += caretPosition.x;
            rect.y += caretPosition.y;
            rect.width = 1;
            rect.height = caretPosition.height;
        }

        let popupSpace = getParentWithFilter(referenceElement, n => n.classList && n.classList.contains("popupSpace"));
        if (popupSpace) {
            let popupSpaceRect = popupSpace.getBoundingClientRect();
            rect.x -= popupSpaceRect.x;
            rect.y -= popupSpaceRect.y;
        }

        popupElement.style.left = rect.x + "px";
        popupElement.style.top = rect.y + "px";
        popupElement.style.width = rect.width + "px";
        popupElement.style.height = rect.height + "px";

        let tooltip = popupElement.getElementsByClassName("tooltip")[0];
        if (tooltip && popupSpace) {
            let popupSpaceRect = popupSpace.getBoundingClientRect();
            tooltip.style.top = "unset";
            let tooltipRect = tooltip.getBoundingClientRect();
            let maxY = popupSpaceRect.height - tooltipRect.height - 3;
            if (tooltipRect.y > maxY) {
                tooltip.style.top = `${(maxY - tooltipRect.y).toFixed(3)}px`;
            }
        }
    }
};

var popupElements = [];

var inputFloatRectangle, inputFloatElement, inputFloatMode;

window.resetInputFloatRectangle = () => {
    inputFloatRectangle = null;
    inputFloatElement = null;
    inputFloatMode = null;
};

window.setInputFloatRectangle = (element, mode) => {
    // mode = "fill" (default) or "below"
    inputFloatRectangle = element;
    inputFloatMode = mode;

    updateInputFloat();
};

window.setInputFloatElement = element => {
    inputFloatElement = element;

    setTimeout(updateInputFloat, 100);
};

function updateInputFloat() {
    if (inputFloatElement && !isAnscestorOf(document.body, inputFloatElement))
        inputFloatElement = null;
    if (inputFloatRectangle && !isAnscestorOf(document.body, inputFloatRectangle))
        inputFloatRectangle = null;

    if (inputFloatElement) {
        inputFloatElement.classList.remove("hidden");
        inputFloatElement.classList.add("shown");

        let visibleRect = getVisibleBoundingRect(inputFloatRectangle);
        let inputControlWrapper = firstOrDefault(inputFloatElement.getElementsByClassName("inputControlWrapper"));
        if (visibleRect.isEmpty || !inputControlWrapper) {
            inputFloatElement.classList.add("movable");
            inputFloatElement.classList.remove("fixed");
            if (inputControlWrapper) {
                inputControlWrapper.style.width = "unset";
                inputControlWrapper.style.height = "unset";
            }
        }
        else {
            inputFloatElement.classList.remove("movable");
            inputFloatElement.classList.add("fixed");
            inputControlWrapper.style.left = visibleRect.left + "px";
            if (inputFloatMode === "below") {
                inputControlWrapper.style.top = visibleRect.bottom + "px";
                inputControlWrapper.style.minWidth = Math.max(60, Math.min(document.body.clientWidth - inputControlWrapper.clientLeft, getRequestedWidth(inputControlWrapper))) + "px";
                inputControlWrapper.style.minHeight = "unset";
                inputControlWrapper.classList.add("input-below");
            }
            else {
                inputControlWrapper.style.top = `${visibleRect.top}px`;
                inputControlWrapper.style.width = Math.max(visibleRect.width, Math.min(document.body.clientWidth - inputControlWrapper.clientLeft, getRequestedWidth(inputControlWrapper))) + "px";
                inputControlWrapper.style.height = visibleRect.height + "px";
                inputControlWrapper.classList.remove("input-below");
            }
        }

        if (inputControlWrapper) {
            let label = firstOrDefault(inputControlWrapper.children, n => n.nodeName === "LABEL");
            if (label) {
                let textBoxWrapper = firstOrDefault(label.children, n => n.classList && n.classList.contains("textBoxWrapper"));
                if (textBoxWrapper) {
                    let element = firstOrDefault(textBoxWrapper.children, n => n.nodeName === "TEXTAREA" || n.nodeName === "INPUT");
                    if (element)
                        updateTextboxSize(element);
                }
            }
        }

        setTimeout(updatePopups);
    }
}

function getRequestedWidth(element) {
    let elementRect = element.getBoundingClientRect();
    let input = element.getElementsByTagName("input")[0];
    if (input) {
        let inputRect = input.getBoundingClientRect();
        return elementRect.width - inputRect.width + getTextWidth(input) + 20;
    }

    return 0;
}

var canvasForMeasuring = document.createElement("canvas");

function getTextWidth(input, textIndex) {
    let canvasContext = canvasForMeasuring.getContext("2d");
    let style = window.getComputedStyle(input);
    canvasContext.font = style.fontSize + " " + style.fontFamily;
    let text = input.value;
    if (typeof textIndex === "number")
        text = text.substring(0, textIndex);
    return canvasContext.measureText(text).width;
}

window.scrollToElement = function (element) {
    if (element) {
        let rect = element.getBoundingClientRect();
        let scrollParent = getScrollParent(element);
        if (scrollParent) {
            let parentRect = scrollParent.getBoundingClientRect();
            if (rect.y < parentRect.y) {
                scrollParent.scrollTop -= parentRect.y - rect.y;
            }
            else if (rect.bottom > parentRect.bottom) {
                scrollParent.scrollTop += rect.bottom - parentRect.bottom;
            }
        }
        else
            return rect;
    }
};

window.getVisibleBoundingRect = element => {
    if (element?.parentNode) {
        let rect = element.getBoundingClientRect();
        for (var anscestor = element.parentNode; anscestor; anscestor = anscestor.parentNode) {
            if (anscestor === document.body ||
                anscestor?.classList?.contains("menuContainer") ||
                anscestor?.classList?.contains("inputControlWrapper"))
                break;

            let anscestorRect = anscestor.getBoundingClientRect();
            rect = getIntersection(rect, anscestorRect);

            if (rect.isEmpty)
                break;

            if (anscestor.classList?.contains("menu"))
                break;
        }
        return rect;
    }
    else {
        return ({ isEmpty: true, width: 0, height: 0 });
    }
}

function getScrollParent(element) {
    return getParentWithFilter(element, n => {
        let style = getComputedStyle(n);
        return style.overflowX === "auto" || style.overflowY === "auto" || n === document.body;
    });
}

function getIntersection(a, b) {
    let left = Math.max(a.x, b.x);
    let top = Math.max(a.y, b.y);
    let right = Math.min(a.right, b.right);
    let bottom = Math.min(a.bottom, b.bottom);
    if (right > left && bottom > top)
        return ({ isEmpty: false, y: top, top: top, x: left, left: left, width: right - left, height: bottom - top, bottom: bottom, right: right });
    else
        return ({ isEmpty: true, width: 0, height: 0 });
}

window.addEventListener("scroll", e => {
    setTimeout(() => {
        updateInputFloat();
        updatePopups();
    });
}, true);

window.addEventListener("resize", e => {
    updateInputFloat();
    updatePopups();
});

window.addEventListener("input", e => {
    if (isAnscestorOf(inputFloatElement, e.target)) {
        updateInputFloat();
        updatePopups();
    }
});

window.positionMovablePopups = function () {
    let popupContainer = document.getElementById("popupContainer");
    for (let child of popupContainer.childNodes) {
        if (child.classList && child.classList.contains("popup") && child.classList.contains("movable")) {
            if (child.style.visibility !== "visible") {
                child.style.visibility = "visible";
                if (child.classList.contains("center-screen")) {
                    let width = child.clientWidth;
                    let height = child.clientHeight;
                    child.style.left = `${(popupContainer.clientWidth - width) * 0.5}px`;
                    child.style.top = `${(popupContainer.clientHeight - height) * 0.5}px`;
                }
                else {
                    let x = 160;
                    let y = 120;
                    for (let i = 0; i < 12; i++) {
                        if (!isMovablePopupNear(x, y)) {
                            child.style.left = `${x}px`;
                            child.style.top = `${y}px`;
                            break;
                        }
                        x += 60;
                        y += 15;
                        if (x > 400 || x > 0.5 * document.body.clientWidth) {
                            x = 160;
                        }
                        if (y > 140 || y > 0.5 * document.body.clientHeight) {
                            x = 160;
                            y = 120;
                            child.style.left = `${x}px`;
                            child.style.top = `${y}px`;
                            break;
                        }
                    }
                }
            }

            // to do: make sure popup is partially overlapping visible area
        }
    }
};

function isMovablePopupNear(x, y) {
    let popupContainer = document.getElementById("popupContainer");
    for (let child of popupContainer.childNodes) {
        if (child.classList && child.classList.contains("popup") && child.classList.contains("movable")) {
            if (child.style.visibility === "visible") {
                let left = child.style.left;
                let top = child.style.top;
                if (left?.endsWith("px") && top?.endsWith("px")) {
                    let _x = Number(left.substring(0, left.length - 2));
                    let _y = Number(top.substring(0, top.length - 2));
                    if (!Number.isNaN(_x) && !Number.isNaN(_y)) {
                        if (Math.abs(_x - x) < 10 && Math.abs(_y - y) < 10) {
                            return true;
                        }
                    }
                }
            }
        }
    }

    return false;
}