﻿import "./pointers.js";

var tableElements = [];

window.setupTableWithBlazor = (element, owner, initialItemCount) => {
    setupTable(element,
        () => owner.invokeMethodAsync("GetFields"),
        (start, count) => owner.invokeMethodAsync("GetItems", start, count),
        (items, fields, eventType, isFirstEvent) => owner.invokeMethodAsync("Select", items, fields, eventType, isFirstEvent),
        null,
        {
            initialItemCount: initialItemCount || 50
        }
    );
};

window.setupTableWithUrl = (element, url, onSelect, getInitialSelection, settings) => {
    setupTable(element,
        () => new Promise((resolve, reject) => {
            let infoUrl = url;
            if (infoUrl.includes("?"))
                infoUrl += "&type=info";
            else
                infoUrl += "?type=info";
            var httpRequest = new XMLHttpRequest();
            httpRequest.open("GET", infoUrl);
            httpRequest.onload = () => {
                if (httpRequest.status >= 200 && httpRequest.status < 300) {
                    let info = JSON.parse(httpRequest.responseText);
                    resolve(info.fields);
                }
                else
                    reject();
            };
            httpRequest.onerror = () => {
                reject();
            };
            httpRequest.send();
        }),
        (start, count) => new Promise((resolve, reject) => {
            let infoUrl = url;
            if (infoUrl.includes("?"))
                infoUrl += `&start=${start}&count=${count}`
            else
                infoUrl += `?start=${start}&count=${count}`
            var httpRequest = new XMLHttpRequest();
            httpRequest.open("GET", infoUrl);
            httpRequest.onload = () => {
                if (httpRequest.status >= 200 && httpRequest.status < 300) {
                    let data = JSON.parse(httpRequest.responseText);
                    resolve(data);
                }
                else
                    reject();
            };
            httpRequest.onerror = () => {
                reject();
            };
            httpRequest.send();
        }),
        onSelect,
        getInitialSelection,
        settings
    );
};

window.getTableElement = filter => {
    return firstOrDefault(tableElements, filter);
};

window.resetTable = (element) => {
    if (element.resetTable)
        element.resetTable();
};

window.updateTableValues = (element) => {
    if (element.queueUpdateTable)
        element.queueUpdateTable();
};

window.addEventListener("scroll", event => {
    for (let i = tableElements.length - 1; i >= 0; i--) {
        let element = tableElements[i];
        if (isSelfOrAnscestorOf(event.target, element)) {
            element.queueUpdateTable();
        }
        else if (!isSelfOrAnscestorOf(document.body, element)) {
            tableElements.splice(i, 1);
        }
    }
}, true);

window.addEventListener("resize", event => {
    for (let i = tableElements.length - 1; i >= 0; i--) {
        let element = tableElements[i];
        if (element.queueUpdateTable)
            element.queueUpdateTable();
    }
});

var selecting;

var resizingGrip = null, resizingPointerId = -1, resizingGripStartX = -1;

window.addEventListener("pointerdown", event => {
    if (event.button !== 0)
        return;

    for (let i = tableElements.length - 1; i >= 0; i--) {
        let element = tableElements[i];
        if (element && element.selectItems && isSelfOrAnscestorOf(element.parentNode, event.target)) {
            let link = getSelfOrParentWithFilter(event.target, n => n.nodeName === "A");
            if (link)
                return;

            let blog = getSelfOrParentWithFilter(element, n => n.nodeName === "DIV" && n.classList.contains("blog"));
            if (blog)
                return;

            let tr = getSelfOrParentWithFilter(event.target, n => n.nodeName === "TR");
            if (tr) {
                let itemValue = tr.getAttribute("row-value");
                let itemIndex = Number.parseInt(tr.getAttribute("row-index"));
                let lastIndex = element.lastClickedIndex;
                element.lastClickedIndex = itemIndex;
                if (itemValue) {
                    event.preventDefault();
                    if (window.isShiftDown) {
                        let thisIndex = itemIndex;
                        let prevItems = element.selectedRowValues.map(i => i);
                        if (lastIndex < thisIndex) {
                            let count = thisIndex - lastIndex + 1;
                            if (count > 100)
                                count = 100;
                            element.getData(lastIndex, count).then(data => {
                                element.selectItems(data.items, null, "pointerDown", true);
                            });
                        }
                        else if (lastIndex > thisIndex) {
                        }
                    }
                    else if (window.isCtrlDown) {

                    }
                    else {
                        selecting = {
                            pointerMove: () => { },
                            pointerUp: e => {
                                let isDbl = window.clickCount === 2;
                                if (Math.abs(e.clientX - event.clientX) < 5 && Math.abs(e.clientY - event.clientY) < 5) {
                                    element.selectItems([JSON.parse(itemValue)], null, isDbl ? "doubleClick" : "click", !isDbl);
                                }
                            }
                        };
                    }
                }
            }

            if (!event.defaultPrevented) {
                element.selectItems([], null, clickCount > 1);
            }
        }
    }

    if (event.target?.classList?.contains("resize-grip")) {
        resizingGrip = event.target;
        resizingGrip.classList.add("resizing");
        resizingGrip.setPointerCapture(event.pointerId);
        resizingPointerId = event.pointerId;
        resizingGripStartX = event.clientX;
    }
});

window.addEventListener("pointermove", event => {
    if (selecting) {
        selecting.pointerMove();
    }
});

window.addEventListener("pointerup", event => {
    if (selecting) {
        selecting.pointerUp(event);
        selecting = null;
    }

    if (resizingGrip && resizingPointerId == event.pointerId) {
        let th = getSelfOrParentWithFilter(resizingGrip, n => n.nodeName === "TH");
        let table = getSelfOrParentWithFilter(th, n => n.nodeName === "TABLE");
        resizingGrip.releasePointerCapture(resizingPointerId);
        resizingGrip.classList.remove("resizing");
        resizingGrip = null;
        resizingPointerId = -1;
        if (th && table && event.clientX !== resizingGripStartX) {
            let newWidth = Math.max(48, th.clientWidth + event.clientX - resizingGripStartX);
            let diff = newWidth - th.clientWidth;
            let newTableWidth = table.clientWidth + diff;
            th.style.width = `${newWidth}px`;
            table.style.width = `${newTableWidth}px`;
        }
    }
});

function setupTable(element, getFields, getData, selectItems, getInitialSelection, settings) {
    if (!tableElements.includes(element))
        tableElements.push(element);

    element.tableControlSettings = {
        selectItems: selectItems,
        getInitialSelection: getInitialSelection,
        settings: settings
    };

    if (!settings)
        settings = {};
    applyDefaults(settings);

    let thead = element.getElementsByTagName("thead")[0];
    let tbody = element.getElementsByTagName("tbody")[0];
    let footer, itemCountDisplay;

    if (element.nextElementSibling && element.nextElementSibling.classList && element.nextElementSibling.classList.contains("tableFooter")) {
        footer = element.nextElementSibling;
        itemCountDisplay = footer.getElementsByClassName("itemCountDisplay")[0];
    }

    let blog = getSelfOrParentWithFilter(element, n => n.tagName === "DIV" && n.classList.contains("blog"));

    let reset = () => {
        let _fields = null;
        let _itemCount = settings.initialItemCount;
        let rowHeights = {};
        let avgRowHeight = settings.avgRowHeight;

        virtualDom.apply([{
            type: "tr", children: []
        }], tbody);

        getFields().then(fields => {
            _fields = fields.map(f => f.title);
            virtualDom.apply(
                [{
                    type: "tr", children:
                        [{ type: "th", children: [{ type: "span", children: ["-"] }] }].concat(fields.map(field => ({ type: "th", children: [{ type: "span", children: [field.title] }, { type: "span", properties: { "class": "resize-grip" } }], properties: getFieldHeaderProperties(field) })))
                }],
                thead
            );
            setTimeout(update, 0);
            setTimeout(() => resetTableLayout(element), 0);
        });

        let cachedData = {};

        let getDataWithCache = async (firstRow, rowCount) => {
            let items = [];
            let totalItemCount = _itemCount;

            let r = firstRow;
            let rowAfter = firstRow + rowCount;
            while (r < rowAfter) {
                let cachedItem = cachedData[r];
                if (cachedItem) {
                    items.push(cachedItem);
                    r++;
                }
                else {
                    let firstRowToGet = r;
                    r++;
                    while (r < rowAfter) {
                        cachedItem = cachedData[r];
                        if (cachedItem) {
                            break;
                        }
                        r++;
                    }
                    let dataResult = await getData(firstRowToGet, r - firstRowToGet);
                    for (let i = 0; i < dataResult.items.length; i++) {
                        let item = dataResult.items[i];
                        items.push(item);
                        cachedData[i + firstRowToGet] = item;
                    }
                    totalItemCount = dataResult.totalItemCount;
                }
            }

            return ({ items: items, totalItemCount: totalItemCount });
        };

        let update = () => {
            updatePending = false;

            let tbodyBounds = tbody.getBoundingClientRect();
            let top = -tbodyBounds.y;
            let bottom = top + window.innerHeight;

            let y = 0;
            let r = 0;
            for (; r < _itemCount; r++)
                if (y > top - 100)
                    break;
                else
                    y += getRowHeight(r);
            let newFirstRow = r - 1;
            for (; r < _itemCount; r++)
                if (y > bottom + 100)
                    break;
                else
                    y += getRowHeight(r);
            let newLastRow = r - 1;
            if (newFirstRow < 0)
                newFirstRow = 0;
            let newCount = newLastRow - newFirstRow + 1;

            if (newFirstRow < firstActiveRow || newLastRow >= firstActiveRow + activeData.length) {
                console.log(`Loading table data at index = ${newFirstRow}, count = ${newCount}`);

                gettingData = true;

                getDataWithCache(newFirstRow, newCount).then(dataResult => {
                    setTimeout(() => {
                        gettingData = false;
                        saveRowHeights();
                    }, 0);

                    if (dataResult.items) {
                        activeData = dataResult.items;
                        firstActiveRow = newFirstRow;
                    }
                    _itemCount = dataResult.totalItemCount;

                    if (itemCountDisplay) {
                        itemCountDisplay.innerHTML = `${_itemCount} items`;
                    }

                    applyRowData();
                }).catch(() => {
                    gettingData = false;
                });
            }
        };

        element.getData = getData;

        element.selectedRowValues = [];

        let activeData = [];
        let firstActiveRow = 0;

        let applyRowData = () => {
            if (getInitialSelection && !any(element.selectedRowValues)) {
                try {
                    let newSelection = getInitialSelection(activeData);
                    if (typeof newSelection === "object" && newSelection.length)
                        element.selectedRowValues = newSelection.map(v => JSON.stringify(v));
                }
                catch { }
            }

            let rowDefs = [{
                type: "tr", properties: { "class": "fillerRow", "style": `height: ${getHeightFrom(0, firstActiveRow)}px; ` }, children: [
                    { type: "td", properties: { "style": "width: 0; height: 0; padding: 0; border: none; " } }
                ]
            }];
            let rowIndex = firstActiveRow;
            activeData.forEach(r => {
                let rowValue = JSON.stringify(r);
                let isSelected = any(element.selectedRowValues, v => v === rowValue);
                let rowChildren;
                let rowClass = isSelected ? "selected" : "";
                let draggable = "true";
                if (r._isGroup) {
                    rowChildren = [
                        {
                            type: "td",
                            children: [{ type: "span" }]
                        },
                        {
                            type: "td",
                            children: [{ type: "h5", children: [r.title] }]
                        }
                    ];
                    rowClass += " grouping";
                    draggable = "false";
                }
                else {
                    rowChildren = [{ type: "td", children: [{ type: "span" }] }].concat(_fields.map(f => ({
                        type: "td",
                        properties: getCellProperties(f, r),
                        children: getValueChildren(r[f])
                    })))
                }
                if (r._linkUrl) {
                    let titleCell = rowChildren[1];
                    if (titleCell && titleCell.type === "td") {
                        titleCell.children = [{
                            type: "a",
                            properties: {
                                "href": r._linkUrl
                            },
                            children: titleCell.children
                        }];
                    }
                }
                let item = {
                    type: "tr", properties: { "row-index": rowIndex.toString(), "row-value": rowValue, "class": rowClass, "draggable": draggable }, children: rowChildren
                };
                if (blog) {
                    item.properties.draggable = undefined;
                }
                rowDefs.push(item);
                rowIndex++;
            });
            rowDefs.push({ type: "tr", properties: { "style": `height: ${getHeightFrom(rowIndex, _itemCount)}px` } });
            virtualDom.apply(
                rowDefs,
                tbody
            );
        };

        let getCellProperties = (f, r) => {
            if (f == _fields[0] && typeof r._indent === "number") {
                let adjIndent = r._indent;
                if (adjIndent > 6)
                    adjIndent = 6;
                return ({
                    class: `indent-${adjIndent}`
                });
            }
        };

        let getRowHeight = r => {
            let v = rowHeights[r.toString()];
            if (v) {
                if (v < 10)
                    return 10;
                return v;
            }
            else
                return avgRowHeight;
        };

        let getHeightFrom = (from, to) => {
            let r = 0;
            for (let i = from; i < to; i++)
                r += getRowHeight(i);
            return r;
        };

        let saveRowHeights = () => {
            let count = 0;
            let cum = 0;
            tbody.childNodes.forEach(tr => {
                let rowIndex = tr.getAttribute("row-index");
                if (rowIndex) {
                    rowHeights[rowIndex] = tr.clientHeight;
                    cum += tr.clientHeight;
                    count++;
                }
            });
            if (count > 0) {
                avgRowHeight = cum / count;
                if (avgRowHeight < 4)
                    avgRowHeight = 4;
                else if (avgRowHeight > 200)
                    avgRowHeight = 200;
            }
        };

        let updatePending = true, gettingData = false;

        element.queueUpdateTable = () => {
            if (!updatePending) {
                updatePending = true;
                tryUpdateAfterDelay();
            }
        };

        let tryUpdateAfterDelay = () => {
            setTimeout(() => {
                if (gettingData)
                    tryUpdateAfterDelay();
                else
                    update();
            }, 200);
        };

        element.selectItems = (items, fields, eventType, isFirstEvent) => {
            if (!eventType)
                isFirstEvent = true;

            if (isFirstEvent) {
                if (items && items.map)
                    element.selectedRowValues = items.map(i => JSON.stringify(i));
                else
                    element.selectedRowValues = null;

                if (applyRowData)
                    applyRowData();
            }

            if (selectItems)
                selectItems(items, fields, eventType, isFirstEvent);
        };
    };

    element.resetTable = reset;

    element.lastClickedIndex = -1;

    reset();
}

function resetTableLayout(element) {
    let thead = element.getElementsByTagName("thead")[0];
    let tr = thead?.getElementsByTagName("tr")[0];
    if (tr && tr.clientWidth > 0) {
        element.style.tableLayout = "auto";
        element.style.width = "unset";

        let tableWidth = element.getBoundingClientRect().width;
        let headerCells = Array.from(tr.getElementsByTagName("th"));
        let headerWidths = Array.from(headerCells.map(th => th.getBoundingClientRect().width));
        for (let i = 0; i < headerCells.length; i++) {
            let th = headerCells[i];
            th.style.width = `${headerWidths[i]}px`;
        }
        element.style.width = `${tableWidth}px`;
        element.style.tableLayout = "fixed";
    }
}

function getFieldHeaderProperties(field) {
    if (typeof field.width === "number") {
        return ({ "data-given-width": `${field.width}`, style: `width: ${field.width}px; ` });
    }
    else {
        return null;
    }
}

function getValueChildren(value) {
    if (typeof value === "string") {
        if (value.startsWith("[") || value.startsWith("{") || value.startsWith('"')) {
            try {
                obj = JSON.parse(value);
            }
            catch (ex) { }
        }
        if (typeof value === "string") {
            if (value === "")
                return ["\u00A0"];
            else
                return [value];
        }
    }

    if (typeof value === "number") {
        return [value.toString()];
    }
    else if (typeof value === "object" && value && value.type === "number") {
        return [`${value.magnitude} ${value.unit}`];
    }
    else if (typeof value === "object" && value && value.type == "icon-item") {
        return [
            {
                type: "span", properties: { class: "icon-item" }, children: [
                    { type: "img", properties: { src: value.icon } },
                    { type: "span", children: [value.text] }
                ]
            }
        ];
    }
    else if (typeof value === "object" && value && value.type == "link") {
        return [
            {
                type: "a", properties: { href: value.href }, children: getValueChildren(value.content)
            }
        ];
    }
    else if (typeof value === "object" && value && value.type == "html-content") {
        return [
            {
                type: "div", properties: { "innerHTML": value.value }
            }
        ];
    }
    else if (typeof value === "boolean") {
        return [value ? "True" : "False"];
    }
    return [];
}

function applyDefaults(settings) {
    if (!settings.initialItemCount)
        settings.initialItemCount = 50;

    if (!settings.avgRowHeight)
        settings.avgRowHeight = 20;
}