﻿
window.setupTextbox = (component, element) => {
    if (element) {
        element.owner = component;
        element.oninput = e => {
            element.lastSelectionStart = element.selectionStart;
            element.lastSelectionEnd = element.selectionEnd;
            let hasEqualsToggle = getSafe(element, "previousElementSibling", "classList", classList => classList.contains("formulaIcon") && classList.contains("toggle"));
            let text = element.value;
            let selStart = element.selectionStart;
            let selLen = element.selectionEnd - element.selectionStart;
            if (hasEqualsToggle) {
                let equalsToggle = element.previousElementSibling;
                if (equalsToggle.style.display !== "none") {
                    text = "=" + text;
                    selStart++;
                }
            }
            component.invokeMethodAsync("OnInput", text, selStart, selLen);
            setTimeout(() => updateTextboxSize(element));
        };
        element.lastSelectionStart = -1;
        element.lastSelectionEnd = -1;
        function onselect(e) {
            setTimeout(() => {
                if (element.lastSelectionStart !== element.selectionStart || element.lastSelectionEnd !== element.selectionEnd) {
                    element.lastSelectionStart = element.selectionStart;
                    element.lastSelectionEnd = element.selectionEnd;
                    let selStart = element.selectionStart;
                    let selLen = element.selectionEnd - element.selectionStart;
                    let hasEqualsToggle = getSafe(element, "previousElementSibling", "classList", classList => classList.contains("formulaIcon") && classList.contains("toggle"));
                    if (hasEqualsToggle) {
                        let equalsToggle = element.previousElementSibling;
                        if (equalsToggle.style.display !== "none") {
                            selStart++;
                        }
                    }
                    component.invokeMethodAsync("OnSelect", selStart, selLen);
                }
            });
        }
        element.onpointerdown = e => onselect();
        element.onpointerup = e => onselect();
        element.onkeydown = e => {
            if (!e.defaultPrevented)
                onselect();
        };
    }
};

window.setTextboxTextAndFocus = (element, text, caretPosition) => {
    if (element) {
        let hasEqualsToggle = getSafe(element, "previousElementSibling", "classList", classList => classList.contains("formulaIcon") && classList.contains("toggle"));

        if (caretPosition < 0) {
            caretPosition = text.length;
        }

        if (hasEqualsToggle) {
            let equalsToggle = element.previousElementSibling;
            if (text.length > 0 && text[0] === "=") {
                equalsToggle.style.display = "flex";
                text = text.substring(1);
                caretPosition--;
            }
            else {
                equalsToggle.style.display = "none";
            }
        }

        element.value = text;
        element.lastSelectionStart = caretPosition;
        element.lastSelectionEnd = caretPosition;
        element.selectionStart = caretPosition;
        element.selectionEnd = caretPosition;
        element.focus();
        updateTextboxSize(element);
    }
};

window.focusElement = (element, selStart, selLen) => {
    if (element && element.focus) {
        element.focus();
        if (selStart < 0 || selLen < 0) {
            selStart = 0;
            selLen = element.value.length;
        }
        else if (selStart + selLen > element.value.length) {
            selLen = element.value.length - selStart;
        }

        element.lastSelectionStart = selStart;
        element.lastSelectionEnd = selStart + selLen;
        element.selectionStart = selStart;
        element.selectionEnd = selStart + selLen;
    }
};

window.updateTextboxSize = element => {
    let textbox = getParentWithFilter(element, n => n.classList && n.classList.contains("textBoxWrapper"));
    if (textbox) {
        let result = firstOrDefault(textbox.children, n => n.classList && n.classList.contains("result"));
        let fixedInputFloat = getParentWithFilter(textbox, n => n.classList && n.classList.contains("floatingInput") && n.classList.contains("fixed"));
        let popupSpace = getParentWithFilter(fixedInputFloat, n => n.classList && n.classList.contains("popupSpace"));
        if (fixedInputFloat) {
            let wrapper = getParentWithFilter(textbox, n => n.classList && n.classList.contains("inputControlWrapper"));
            let textboxRect = textbox.getBoundingClientRect();
            let availRect = { x: textboxRect.x, y: textboxRect.y, width: popupSpace.clientWidth - textboxRect.x - 4, height: popupSpace.clientHeight - textboxRect.y - 4 };
            let textSize = measureWrappedText(element, availRect.width);
            let minWidth = wrapper.clientWidth;
            let minHeight = wrapper.clientHeight;
            let isBelow = wrapper.classList.contains("input-below");
            if (isBelow) {
                minWidth = 40;
                minHeight = 20;
            }
            if (element.nodeName === "TEXTAREA") {
                let width = textSize.width + 10;
                if (result) {
                    result.style.display = "block";
                    width += result.clientWidth;
                }
                else {
                    width += 0;
                }
                if (width < minWidth)
                    width = minWidth;
                else if (width > availRect.width)
                    width = availRect.width;
                let height = textSize.height;
                if (height < minHeight)
                    height = minHeight;
                else if (height > availRect.height)
                    height = availRect.height;
                textbox.style.height = `${height}px`;
                textbox.style.width = `${width}px`;
            }
            else if (element.nodeName === "INPUT") {

            }
        }
        else if (getParentWithFilter(textbox, n => n.classList && n.classList.contains("floatingInput"))) {
            let size = measureWrappedText(element, element.clientWidth);
            if (element.nodeName === "TEXTAREA") {
                let height = size.height + 4;
                if (height > 140) {
                    height = 140;
                }
                textbox.style.height = `${height}px`;
                textbox.style.width = "unset";

            }
            else if (element.nodeName === "INPUT") {

            }
        }
    }
};

function measureWrappedText(element, availWidth) {
    let div = createDivClone(element, element.value);
    div.style.maxWidth = `${availWidth}px`;
    div.style.width = "auto";
    div.style.overflowY = "auto";
    document.body.appendChild(div);
    let rect = div.getBoundingClientRect();
    document.body.removeChild(div);
    return ({ width: rect.width, height: rect.height });
}

// Given input or textarea element and text position, return object with x, y, and height
window.getCaretPosition = (element, position) => {
    if (element.nodeName === "TEXTAREA" || element.nodeName === "INPUT") {
        let hasEqualsToggle = getSafe(element, "previousElementSibling", "classList", classList => classList.contains("formulaIcon") && classList.contains("toggle"));
        if (hasEqualsToggle && position > 0)
            position--;
        let div = createDivClone(element, element.value.substring(0, position));
        document.body.appendChild(div);
        let lastLine = lastOrDefault(div.children, n => n.nodeName === "DIV");
        let lastLineRect = lastLine.getBoundingClientRect();
        let divRect = div.getBoundingClientRect();
        document.body.removeChild(div);
        return ({ x: lastLineRect.right - divRect.x - element.scrollLeft, y: lastLineRect.y - divRect.y - element.scrollTop, height: lastLineRect.height });
    }

    return ({ x: 0, y: 0, height: element.clientHeight });
};

// For temporary metrics; in future, use this as basis for custom rendering
function createDivClone(element, text) {
    let lines = text.split(/\r?\n/);
    let div = document.createElement("div");
    let style = getComputedStyle(element);
    div.style.paddingLeft = style.paddingLeft;
    div.style.paddingTop = style.paddingTop;
    div.style.paddingRight = style.paddingRight;
    div.style.paddingBottom = style.paddingBottom;
    div.style.width = `${element.clientWidth}px`;
    div.style.overflowY = style.overflowY;
    div.style.overflowX = style.overflowX;
    div.style.fontFamily = style.fontFamily;
    div.style.fontSize = style.fontSize;
    div.style.position = "absolute";
    div.style.visibility = "hidden";
    div.style.display = "flex";
    div.style.flexDirection = "column";
    div.style.alignItems = "start";
    for (let line of lines) {
        let lineDiv = document.createElement("div");
        lineDiv.innerText = line;
        lineDiv.style.whiteSpace = "pre";
        lineDiv.style.display = "inline-block";
        if (line.length === 0) {
            lineDiv.innerText = " ";
            lineDiv.style.width = "10px";
            lineDiv.style.position = "relative";
            lineDiv.style.left = "-10px";
        }
        div.appendChild(lineDiv);
    }
    return div;
}