﻿let currentToolTip = null;

let pointerX = 0, pointerY = 0;

window.addEventListener("pointermove", event => {
    pointerX = event.clientX;
    pointerY = event.clientY;
}, true);

window.addEventListener("pointerdown", event => {
    hideToolTip();
    pendingToolTip = null;
});

let timer = null;
let pendingToolTip = null;

window.showFloatingToolTip = (wrapperElement, referenceElement, placement) => {
    if (wrapperElement) {
        wrapperElement.referenceElement = referenceElement;

        let toolTip = {
            referenceElement: referenceElement,
            wrapperElement: wrapperElement
        };
        updatePopup(toolTip.wrapperElement);
        pendingToolTip = toolTip;
        if (currentToolTip) {
            showToolTipNow(toolTip);
        } else {
            setTimeout(() => showToolTipNow(toolTip), 500);
        }
    }
};

function showToolTipNow(toolTip) {
    if (toolTip?.referenceElement !== pendingToolTip?.referenceElement)
        return;

    pendingToolTip = null;

    if (!pointerOverElement(toolTip.referenceElement))
        return;

    toolTip.wrapperElement.style.display = "block";
    toolTip.wrapperElement.classList.add("open");
    toolTip.wrapperElement.classList.remove("closed");
    if (!timer) {
        timer = setInterval(checkToHideToolTip, 40);
    }
    currentToolTip = toolTip;
}

function checkToHideToolTip() {
    let referenceElement = currentToolTip?.referenceElement;
    if (referenceElement) {
        if (!pointerOverElement(referenceElement)) {
            hideToolTip();
        }
    }
}

function hideToolTip() {
    if (!currentToolTip?.wrapperElement) {
        clearInterval(timer);
        timer = null;
        return;
    }

    let referenceElement = currentToolTip.referenceElement;
    currentToolTip.wrapperElement.classList.remove("open");
    currentToolTip.wrapperElement.classList.add("closed");
    clearInterval(timer);
    timer = null;
    currentToolTip.hidden = true;

    setTimeout(() => {
        if (referenceElement === currentToolTip?.referenceElement && currentToolTip.hidden)
            currentToolTip = null;
    }, 500);
}

function pointerOverElement(element) {
    let rect = element.getBoundingClientRect();

    return rect.left <= pointerX && rect.right >= pointerX && rect.top <= pointerY && rect.bottom >= pointerY;
}

window.anyLabelKeptVisible = elements => {
    for (let element of elements) {
        let rect = element.getBoundingClientRect();
        if (rect.x - 20 <= pointerX && rect.y - 20 <= pointerY && rect.right + 20 >= pointerX && rect.bottom + 20 >= pointerY) {
            return true;
        }
    }

    return false;
};