﻿
window.virtualDom = (() => {
    /* item in format {type: tag name, properties: , children: [...], onAdding: element => ..., onRemoving: element => ... } */

    function testExisting(item, existing) {
        if (typeof item === "object")
            return item === existing.model || item.type === existing.nodeName;
        else if (typeof item === "string")
            return item.nodeType === "#text";
        else
            return false;
    }

    function updateExisting(item, existing) {
        applyAttributes(item, existing);
        if (whetherToApplyChildren(item))
            applyItems(item.children, existing);
    }

    function createNew(item) {
        if (typeof item === "object" && item.type) {
            var r = document.createElement(item.type);
            r.model = item;
            applyAttributes(item, r);
            if (whetherToApplyChildren(item))
                applyItems(item.children, r, true);
            return r;
        }
        else if (typeof item === "string") {
            return document.createTextNode(item);
        }
        else {
            return document.createElement("span");
        }
    }

    function applyAttributes(item, element) {
        if (element && element.attributes) {
            let remainingAtts = {};
            for (let i = 0; i < element.attributes.length; i++) {
                remainingAtts[element.attributes[i].name] = {};
            }
            if (item.properties) {
                for (let attDef of Object.keys(item.properties)) {
                    let v = item.properties[attDef];
                    if (attDef === "innerHTML") {
                        if (!item.children) {
                            element.innerHTML = v;
                        }
                    } else {
                        if (typeof v === "string")
                            element.setAttribute(attDef, v);
                        else
                            element[attDef] = v; // is this depended on anywhere? 
                        remainingAtts[attDef] = undefined;
                    }
                }
            }
            for (let remaingingAtt of Object.keys(remainingAtts)) {
                if (remainingAtts[remaingingAtt]) {
                    element.removeAttribute(remaingingAtt);
                }
            }
        }
    }

    function removing(element) {
        if (element.model) {
            if (element.model.onRemoving)
                element.model.onRemoving(element);

            for (let n = element.firstChild; n; n = n.nextSibling)
                removing(n);
        }
    }

    function adding(element) {
        if (element.model) {
            if (element.model.onAdding)
                element.model.onAdding(element);

            for (let n = element.firstChild; n; n = n.nextSibling)
                adding(n);
        }
    }

    function whetherToApplyChildren(item) {
        return !(item && item.properties && item.properties.innerHTML);
    }

    function applyItems(items, destination, silent) {
        if (destination.childNodes) {
            let i = 0;
            if (items && items.length) {
                for (i = 0; i < items.length; i++) {
                    let item = items[i];
                    if (i < destination.childNodes.length) {
                        let existing = destination.childNodes[i];
                        if (testExisting(item, existing))
                            updateExisting(item, existing);
                        else {
                            let newElement = createNew(item);
                            removing(existing);
                            destination.replaceChild(newElement, existing);
                            if (!silent)
                                adding(newElement);
                        }
                    }
                    else if (item) {
                        let newElement = createNew(item);
                        destination.appendChild(newElement);
                        if (!silent)
                            adding(newElement);
                    }
                }
            }
            while (destination.childNodes.length > i) {
                destination.removeChild(destination.childNodes[i]);
            }
        }
    }

    return (
        {
            apply: applyItems
        });
})();
