﻿import { getMainWindowId } from "./helpers.js";

let electron;

let urlParams = new URLSearchParams(window.location.search);
let mainWindowId = getMainWindowId().toString();
let isMac = urlParams.get("mac") === "true";

if (window.require) {
    electron = require("electron");
}

let whenReady = new Promise((resolve, reject) => {
    window.markReady = resolve;
});

window.whenBlazorReady = whenReady;

if (window.location.pathname === "/main-window") {
    if (electron) {
        electron.ipcRenderer.on("focus", () => {
            DotNet.invokeMethodAsync("Blockpad", "MainWindowFocused", mainWindowId);
        });

        electron.ipcRenderer.on("shown", () => {
            DotNet.invokeMethodAsync("Blockpad", "MainWindowShown", mainWindowId);
        });

        electron.ipcRenderer.on("run-tool", (event, mainWindowId, path) => {
            DotNet.invokeMethodAsync("Blockpad", "MainWindowRunTool", mainWindowId, path);
        });

        electron.ipcRenderer.on("try-close-window", async (event, mainWindowId) => {
            try {
                let hasUnsavedChanges = await DotNet.invokeMethodAsync("Blockpad", "MainWindowHasUnsavedChanges", mainWindowId);
                if (hasUnsavedChanges)
                    electron.ipcRenderer.invoke("show-close-dialog", mainWindowId);
                else
                    electron.ipcRenderer.invoke("close-main-window", mainWindowId);
            }
            catch (ex) {
                electron.ipcRenderer.invoke("close-main-window", mainWindowId);
            }
        });

        electron.ipcRenderer.on("new-tab", async (event, mainWindowId, path) => {
            console.log("Open new tab");
            await whenReady;
            console.log("Ready");
            DotNet.invokeMethodAsync("Blockpad", "MainWindowNewTab", mainWindowId, path);
        });
    }

    window.activateWindow = () => {
        electron.ipcRenderer.invoke("activate-window", mainWindowId);
    };

    window.whetherToMoveTab = (event, lastEvent, tab, allTabs) => {
        // to do: check whether to move off window

        let tabRect = tab.getBoundingClientRect();
        let tabIndex = allTabs.indexOf(tab);

        if (event.clientX < tabRect.x && event.clientX < lastEvent.clientX) {
            for (let i = 0; i < tabIndex; i++) {
                if (event.clientX < allTabs[i].getBoundingClientRect().right - 5) {
                    return ({
                        type: "in-list",
                        number: i
                    });
                }
            }

            return ({
                type: "none"
            });
        }
        else if (event.clientX > tabRect.right && event.clientX > lastEvent.clientX) {
            for (let i = allTabs.length - 1; i > tabIndex; i--) {
                if (event.clientX > allTabs[i].getBoundingClientRect().x + 5) {
                    return ({
                        type: "in-list",
                        number: i
                    });
                }
            }

            return ({
                type: "none"
            });
        }

        return ({
            type: "none"
        });
    };

    let tabCaptured = null;
    let tabCapturePointerId = -1;

    window.addEventListener("pointerdown", e => {
        releaseTabCapture();

        if (e.buttons === 1) {
            let tab = getSelfOrParentWithFilter(e.target, n => n.tagName === "BUTTON" || n.classList?.contains("tab"));
            if (tab?.tagName !== "BUTTON") {
                tab?.setPointerCapture(e.pointerId);
                tabCapturePointerId = e.pointerId;
                tabCaptured = tab;
            }
        }
    });

    window.addEventListener("pointerup", e => {
        releaseTabCapture();
    });

    window.recaptureTabPointer = tab => {
        if (tabCaptured && tabCapturePointerId >= 0) {
            tabCaptured.setPointerCapture(tabCapturePointerId);
        }
    };

    function releaseTabCapture() {
        if (tabCaptured) {
            tabCaptured.releasePointerCapture(tabCapturePointerId);
            tabCaptured = null;
            tabCapturePointerId = -1;
        }
    }

    document.addEventListener("dblclick", event => {
        let target = event.target;
        let style = window.getComputedStyle(target);
        if (style["-webkit-app-region"] === "drag") {
            electron.ipcRenderer.invoke("double-click-titlebar", getMainWindowId().toString());
        }
    });
}

window.addEventListener("keydown", e => {
    let ctrlOrMeta = isMac ? e.metaKey : e.ctrlKey;
    if (e.key === "Tab" && e.ctrlKey) {
        if (e.shiftKey) {
            let mainWindowId = getMainWindowId().toString();
            DotNet.invokeMethodAsync("Blockpad", "MainWindowPrevTab", mainWindowId);
            e.preventDefault();
            e.stopImmediatePropagation();
        }
        else {
            let mainWindowId = getMainWindowId().toString();
            DotNet.invokeMethodAsync("Blockpad", "MainWindowNextTab", mainWindowId);
            e.preventDefault();
            e.stopImmediatePropagation();
        }
    }
    else if (e.key === "w" && ctrlOrMeta && !e.shiftKey) {
        let mainWindowId = getMainWindowId().toString();
        DotNet.invokeMethodAsync("Blockpad", "MainWindowCloseCurrentTab", mainWindowId);
        e.preventDefault();
        e.stopImmediatePropagation();
    }
    else if (e.key === "t" && ctrlOrMeta && !e.shiftKey) {
        let mainWindowId = getMainWindowId().toString();
        DotNet.invokeMethodAsync("Blockpad", "MainWindowNewTab", mainWindowId, null);
        e.preventDefault();
        e.stopImmediatePropagation();
    }
}, true);

window.setMacMenu = menu => {
    electron.ipcRenderer.invoke("set-mac-menu", getMainWindowId(), menu);
};

window.setTabTitle = title => {
    electron.ipcRenderer.invoke("set-tab-title", getMainWindowId(), title);
};

window.showElectronDevTools = () => {
    electron.ipcRenderer.invoke("show-dev-tools", getMainWindowId());
};

window.forceCloseMainWindow = () => {
    electron.ipcRenderer.invoke("close-main-window", getMainWindowId());
};

window.focusOnCurrentTab = () => {
    let tab = getSelfOrParentWithFilter(document.activeElement, n => n.nodeName === "IFRAME" && n.classList.contains("active"));
    if (!tab) {
        let activeTab = firstOrDefault(document.getElementsByTagName("iframe"), n => n.classList.contains("active"));
        activeTab?.contentWindow?.focus();
    }
};

window.addEventListener("click", async e => {
    let link = getSelfOrParentWithFilter(e.target, n => n.nodeName === "A");
    if (link && link.href && !link.href.startsWith("#")) {
        let path = link.href;
        if (path.startsWith("file://"))
            path = path.substring(7);
        if (path.startsWith("/") && path.length > 2 && path[2] == ":")
            path = path.substring(1);
        let protocolAndHost = `${location.protocol}//${location.hostname}:${location.port}/`;
        if (path.startsWith(protocolAndHost))
            path = path.substring(21);
        let mainWindowId = getMainWindowId();
        if (mainWindowId >= 0) {
            DotNet.invokeMethodAsync("Blockpad", "MainWindowFollowLink", mainWindowId.toString(), path);
            e.preventDefault();
        }
        else {
            e.preventDefault();
            let adjustedPath = await DotNet.invokeMethodAsync("Blockpad", "MainWindowGetAdjustedLink", path);
            if (adjustedPath)
                window.open(adjustedPath, "_blank");
        }
    }
});

window.setThemeCss = css => {
    let themeStyle = document.getElementById("theme-style");
    if (themeStyle)
        themeStyle.innerHTML = css;
    let iframes = document.getElementsByTagName("iframe");
    for (let iframe of iframes) {
        themeStyle = iframe.contentDocument.getElementById("theme-style");
        if (themeStyle)
            themeStyle.innerHTML = css;
    }
};

window.newMainWindow = () => {
    electron.ipcRenderer.invoke("new-main-window");
};